@@ -1,7 +1,21 @@
+
 __all__ = [
-    "BasePlot"
+    "NUKE_COLOR",
+    "THERMAL_COLOR",
+    "WIND_COLOR",
+    "SOLAR_COLOR",
+    "HYDRO_COLOR",
+    "NUKE_ID",
+    "THERMAL_ID",
+    "WIND_ID",
+    "SOLAR_ID",
+    "HYDRO_ID",
+    "TYPE_GEN",
+    "COLOR_GEN",
+    "BasePlot",
 ]
 
+from grid2op.PlotGrid.config import *
 from grid2op.PlotGrid.BasePlot import BasePlot
 
 # Conditional exports for optional dependencies
