@@ -15,6 +15,7 @@ __all__ = [
     "CombinedScaledReward",
     "RewardHelper",
     "BaseReward",
+    "EpisodeDurationReward",
     # TODO it would be better to have a specific package for this, but in the mean time i put it here
     "L2RPNSandBoxScore"
 ]
@@ -36,6 +37,7 @@ from grid2op.Reward.CombinedScaledReward import CombinedScaledReward
 from grid2op.Reward.RewardHelper import RewardHelper
 from grid2op.Reward.BaseReward import BaseReward
 from grid2op.Reward.L2RPNSandBoxScore import L2RPNSandBoxScore
+from grid2op.Reward.EpisodeDurationReward import EpisodeDurationReward
 
 import warnings
 
